from typing import Dict, Any

import librosa
import soundfile
import torch
from scipy.io import wavfile
from transformers import Wav2Vec2ForCTC, Wav2Vec2Processor

from sziszapangma.integration.asr_processor import AsrProcessor
from sziszapangma.model.model_creators import create_new_word


class Wav2Vec2AsrProcessor(AsrProcessor):
    _model: Wav2Vec2ForCTC
    _wav2vec2_processor: Wav2Vec2Processor
    _device: str

    def __init__(self, model_name: str = "facebook/wav2vec2-large-xlsr-53-dutch"):
        self._device = 'cuda' if torch.cuda.is_available() else 'cpu'
        self._model = Wav2Vec2ForCTC.from_pretrained(model_name).to(self._device)
        self._wav2vec2_processor = Wav2Vec2Processor.from_pretrained(model_name)

    def call_recognise(self, file_path: str) -> Dict[str, Any]:
        data, samplerate = librosa.load(file_path, sr=16000)
        features = self._wav2vec2_processor(data, sampling_rate=samplerate, padding=True, return_tensors="pt")
        input_values = features.input_values.to(self._device)
        attention_mask = features.attention_mask.to(self._device)
        with torch.no_grad():
            logits = self._model(input_values, attention_mask=attention_mask).logits
        pred_ids = torch.argmax(logits, dim=-1)
        result = self._wav2vec2_processor.batch_decode(pred_ids)[0]
        return {
            "transcription": result.split(),
            "full_text": result,
            "words_time_alignment": None
        }
